#!r6rs
;; FILE: "tiny-tests.ss"
;; IMPLEMENTS: Regression test suite for tiny-code.ss
;; LANGUAGE: Ikarus Scheme
;; AUTHOR: Ken [dot] Dickey [at] Whidbey [dot] Com
;; DATE: 9 January 2008

;; COPYRIGHT (c) 2008 by Kenneth A Dickey.  All rights reserved.

;==============================================================;
(import (rnrs (6))
;;      (tiny-talk) ;; get defs via tiny-talk-plus
        (format)
        (tiny-talk-plus)
        (tiny-code)
        (simple-regression-testing))
;==============================================================;

(remove-all-test-suites) ;; start clean
(start-tiny-talk) 	 ;; init tt & tt+

;==============================================================;
;;; POINTs

;; local values

(define p0 (new-point  0   0))
(define p1 (new-point 23  14))
(define p2 (new-point  7 123))
(define p3 (new-point 17   5))
(define p4 (new-point 20  14))
(define (=? a b) [$ =? a b])
[$ add-variable! p4
   (field-spec z 0
               (lambda (val)
                 (if (integer? val)
                     val
                     (error 'z "new val must be an integer" val))))]
[$ add-method! p4 'point3d? (lambda (self) #t)]

;; tests

(add-test-suite 'point default-setup-thunk default-teardown-thunk)

(add-eq-test 'point #f (point? 3) "(point? 3)")
(add-eq-test 'point #t (point? p1) "(point? p1)")
(add-eq-test 'point #t [$ =?  p1 (new-point [$ x p1] [$ y p1])] "point:=?")
(add-eq-test 'point #f [$ =?  p1 p2] "(point:=? p1 p2)")
(add-eq-test 'point #t [$ <>? p1 p2] "(point:<>? p1 p2)")
(add-eq-test 'point #f [$ <>? p1 (new-point [$ x p1] [$ y p1])] "point:<>?")

(add-eq-test 'point #t [$ point3d? p4] "[$ point3d? p4]")
(add-equal-test 'point  0 [$ z p4]     "[$ z p4]")
(add-equal-test 'point 55 [$ z p4 55]  "[$ z p4 55]")
(add-equal-test 'point 55 [$ z p4]     "[$ z p4]")
(ensure-exception-raised 'point
           (lambda (exn)
             (and (message-condition? exn)
                  (string=? "new val must be an integer"
                            (condition-message exn))))
           [$ z p4 'bogus]
           "[$ z p4 'bogus]")

(ensure-exception-raised 'point
           (lambda (exn)
             (and (message-condition? exn)
                  (string=? "Don't know how compare point to non-point"
                            (condition-message exn))))
           [$ =? p1 3]
           "[$ =? p1 3")
(add-test 'point (new-point 30  137) [$ add p1 p2] =?)
(add-test 'point (new-point 16 -109) [$ sub p1 p2] =?)
(add-test 'point (new-point 26   17) [$ add p1  3] =?)
(add-test 'point (new-point 20   11) [$ sub p1  3] =?)
(add-test 'point (new-point 15   68) [$ point-between p1 p2] =?)
(add-test 'point (new-point -23 -14) [$ negate p1] =?)

(add-eq-test 'point #t [$ <=? p0 p1])
(add-eq-test 'point #f [$ <=? p1 p3])
(add-eq-test 'point #t [$ <?  p0 p1])
(add-eq-test 'point #f [$ <?  p4 p1])
(add-eq-test 'point #t [$ <=? p4 p1])
(add-eq-test 'point #f [$ >?  p4 p1])
(add-eq-test 'point #t [$ >?  p4 p0])
(add-eq-test 'point #t [$ >=? p1 p4])
(add-eq-test 'point #f [$ >?  p1 p4])
(add-eq-test 'point #t [$ <>? p1 p4])
(add-eq-test 'point #f [$ <>? p1 p1])

(add-test 'point (new-point   7   5) [$ min-point p2 p3] =?)
(add-test 'point (new-point  17 123) [$ max-point p2 p3] =?)
(add-test 'point (new-point 120 123) [$ max-point (new-point 120 45) p2] =?)

(add-test 'point 5.0 [$ distance-between (new-point 0 3) (new-point 4 0)] =)

(add-test 'point (new-point 16 13)
          [$ nearest-point-on-line-between p1 p2 p3] =?)

(ensure-exception-raised 'point
           (lambda (exn)
             (and (message-condition? exn)
                  (string=? "Needs a point and two line end-points"
                            (condition-message exn))))
           [$ nearest-point-on-line-between p1 2 p3]
           )

(add-test 'point (new-point 3 4) (polar->point 5 (asin (+ 0.0 (/ 3 5)))) =?)
(add-test 'point (new-rectangle [$ x p1] [$ y p1] [$ x p2] [$ y p2])
                 [$ rect-from-extent p1 p2] =?)
(add-test 'point (new-rectangle 10 20 30 40)
                 [$ rect-from-2-points (new-point 10 20) (new-point 40 60)] =?)
(add-test 'point (new-rectangle [$ x p1] [$ y p1] 0 0)
                 [$ as-rectangle p1] =?)
(add-test 'point 5.0 [$ polar-r (new-point 3 4)] =)
;; NOTE: 2 argument atan NYI for Ikarus at this time. (atan 4 3)
(add-test 'point (atan (+ 0.0 (/ 4 3))) [$ polar-theta (new-point 3 4)] =)

(add-test 'point (new-rectangle 7 5 113 118)
          (list-of-points->rectangle
           (list p1 p2 p3 p4 (new-point 120 24))) =?)



;==============================================================;
;;; RECTANGLEs

(add-test-suite 'rectangle default-setup-thunk default-teardown-thunk)

(define r1 (new-rectangle  3  4 600 400)) ;; (x,y) width height
(define r2 (new-rectangle  4  6 200 100))
(define r3 (new-rectangle  5 10 600 400))
(define r4 (new-rectangle 15  5 600 400))

(add-eq-test 'rectangle #t [$ contains-rect? r1 r2])
(add-eq-test 'rectangle #f [$ contains-rect? r1 r3])
(add-eq-test 'rectangle #t [$ contains-point? r1 (new-point 5 8)])
(add-eq-test 'rectangle #f [$ contains-point? r1 (new-point 2 2)])
(add-test 'rectangle (+ 3 600) [$ max-x r1] =)
(add-test 'rectangle (+ 4 400) [$ max-y r1] =)
(add-test 'rectangle (new-rectangle   3 4 300 400) [$ with-width  r1 300] =?)
(add-test 'rectangle (new-rectangle   3 4 600 200) [$ with-height r1 200] =?)
(add-test 'rectangle (new-rectangle  12 4 600 400) [$ with-x      r1  12] =?)
(add-test 'rectangle (new-rectangle  3 12 600 400) [$ with-y      r1  12] =?)
(add-test 'rectangle (new-rectangle  3  4 100 200)
          	     [$ with-extent r1 (new-point 100 200)] =?)
(add-test 'rectangle (new-point  (+ 3 300) (+ 4 200)) [$ center-point r1] =?)
(add-test 'rectangle (new-point   3   4) [$ top-left-point      r1] =?)
(add-test 'rectangle (new-point 603   4) [$ top-right-point     r1] =?)
(add-test 'rectangle (new-point   3 404) [$ bottom-left-point   r1] =?)
(add-test 'rectangle (new-point 603 404) [$ bottom-right-point  r1] =?)
(add-test 'rectangle (new-point   3 204) [$ left-center-point   r1] =?)
(add-test 'rectangle (new-point 603 204) [$ right-center-point  r1] =?)
(add-test 'rectangle (new-point 303   4) [$ top-center-point    r1] =?)
(add-test 'rectangle (new-point 303 404) [$ bottom-center-point r1] =?)
(add-test 'rectangle (new-rectangle 100 100 503 304)
                     [$ with-top-left      r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-rectangle 3 100  97 304)
                     [$ with-top-right     r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-rectangle 100 4 503  96)
                     [$ with-bottom-left   r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-rectangle 3 4 97 96)
                     [$ with-bottom-right  r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-rectangle 100 4 503 400)
                     [$ with-left-center   r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-rectangle 100 4  97 400)
                     [$ with-right-center  r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-rectangle 3   4 600 96)
                     [$ with-bottom-center r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-rectangle 3 100 600 304)
                     [$ with-top-center    r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-point 100 100)
                     [$ constrain-point    r1 (new-point 100 100)] =?)
(add-test 'rectangle (new-point 3 4)
                     [$ constrain-point    r1 (new-point 1 1)] =?)
(add-test 'rectangle (new-point 3 404)
                     [$ constrain-point    r1 (new-point 1 800)] =?)
(add-test 'rectangle (new-point 603 30)
                     [$ constrain-point    r1 (new-point 800 30)] =?)
(add-test 'rectangle (new-point 603 404)
                     [$ constrain-point    r1 (new-point 800 800)] =?)
(add-test 'rectangle (new-rectangle 15 10 590 395)
                     [$ intersection  r3 r4] =?)
(add-test 'rectangle [$ intersection  r4 r3]
                     [$ intersection  r3 r4] =?)
(add-test 'rectangle (new-rectangle  5  5 610 405)
                     [$ union         r3 r4] =?)
(add-test 'rectangle [$ union         r4 r3]
                     [$ union         r3 r4] =?)
(add-eq-test 'rectangle #t [$ empty? (new-rectangle 2 3 0 0)])
(add-eq-test 'rectangle #f [$ empty? (new-rectangle 2 3 1 0)])
(add-eq-test 'rectangle #f [$ empty? (new-rectangle 2 3 0 1)])
(add-eq-test 'rectangle #t [$ empty? (new-rectangle 2 3 0 0)])
(add-eq-test 'rectangle #t [$ empty? [$ rect-from-2-points p1 p1]])
(add-eq-test 'rectangle #f [$ empty? r1])
(add-test 'rectangle (new-point 200 300)
                     [$ closest-point-to (new-rectangle 200 300 80 40) p1] =?)
(add-test 'rectangle (new-rectangle 26 18 600 400)
                     [$ translate-by r1 p1] =?)
(add-test 'rectangle (new-rectangle    3    4   3000  2000)
                     [$ scale-by r1  5] =?)
(add-test 'rectangle (new-rectangle    3    4  13800  5600)
                     [$ scale-by r1 p1] =?)
(add-test 'rectangle (new-rectangle 2403 2404 120000 40000)
                     [$ scale-by r1 r2] =?)
(add-test 'rectangle (new-rectangle 8 9 590 390)
                     [$ inset-by r1 5] =?)
(add-test 'rectangle (new-rectangle 7 13 596 394)
                     [$ inset-by r3 (new-point 2 3)] =?)
(add-test 'rectangle (new-rectangle 1 6 608 408)
                     [$ expand-by r3 4] =?)
(add-test 'rectangle (new-rectangle 3 7 604 406)
                     [$ expand-by r3 (new-point 2 3)] =?)


;==============================================================;
;;; COLOR

(add-test-suite 'color default-setup-thunk default-teardown-thunk)

(define c1 (color-named 'blue))
(define c2 (rgb->color 12 124 65))

(add-eq-test 'color #t (color? c1))
(add-eq-test 'color #f (color? 'blue))
(add-eq-test 'color #f [$ =? c1 c2])
(add-eq-test 'color #t (=? (color-named 'black)
                           (rgb->color   0   0   0)))
(add-eq-test 'color #t (=? (color-named 'white)
                           (rgb->color 255 255 255)))

(ensure-exception-raised 'color
           (lambda (exn)
             (and (message-condition? exn)
                  (string=? "red, green, and blue values must be in range 0..1"
                            (condition-message exn))))
           (new-color 255 120 15)
           )

(add-test 'color (new-color 0 0 0.8)
                 [$ mixed-with c1 c2 1] =?)
(add-test 'color (new-color 0.875 0.875 0.975)
                 [$ lighter c1 2] =?)
(add-test 'color (new-color 0.0 0.0 0.1)
                 [$ darker  c1 2] =?)
(add-test 'color (new-color 0.7 0.6475 0.49)
                 (hsb->color 45 .3 .7) =?)
(add-test 'color (new-color .5  1 .5)
                 (hsb->color 120 .5 1) =?)
(add-test 'color (new-color  0  0 .5)
                 (hsb->color 240 1 .5) =?)
(add-test 'color (rgb->color  255  0  0)
                 (hsb->color  0  1  1) =?)

;==============================================================;
;;; EVENTs

(add-test-suite 'events default-setup-thunk default-teardown-thunk)

(define e1 (make-event 'mouse-down '(x . 40) '(y . 100)))
(define e2 (make-event 'mouse-move '(x . 45) '(y .  98)))
(define e3 (make-event 'mouse-up   '(x . 46) '(y .  19)))
(define (lookup name alist)
  (cond
   ((assq name alist) => cdr)
   (else (error 'lookup "name not found" name alist))))
(define test-handler
  (lambda (evt)
    (list [$ name evt] (lookup 'x [$ args evt]) (lookup 'y [$ args evt]))))

[$ add-handler! event-handlers 'mouse-up   test-handler]
[$ add-handler! event-handlers 'mouse-down test-handler]
[$ add-handler! event-handlers 'mouse-move test-handler]

(add-equal-test 'events '(mouse-down 40 100)
                [$ handle-event event-handlers e1])
(add-equal-test 'events '(mouse-move 45  98)
                [$ handle-event event-handlers e2])
(add-equal-test 'events '(mouse-up   46  19)
                [$ handle-event event-handlers e3])
(add-test 'events '(mouse-up   46  19)
                [$ handle-event event-handlers e3] equal?)



;==============================================================;
;(import (tiny-talk-plus))
;(start-tiny-talk)

(define (vowel? c) (and (char? c) (memq c (string->list "aeiou")) #t))
(define (nv? x) (not (vowel? x)))
(define (id x) x) ;; identity

(add-test-suite 'tt-plus default-setup-thunk default-teardown-thunk)

(add-equal-test 'tt-plus "abcdef" [$ join "abc" "def"])
(add-equal-test 'tt-plus (vector 1 2 3 'a 'b 'c)
                [$ join (vector 1 2 3) (vector 'a 'b 'c)])
(add-equal-test 'tt-plus (list 1 2 3 'a 'b 'c)
                [$ join '(1 2 3) '(a b c)])
(add-eq-test 'tt-plus #t [$ every? "aeiou" vowel?])
(add-eq-test 'tt-plus #t [$ every? (string->list "aeiou") vowel?])
(add-eq-test 'tt-plus #t [$ every? (list->vector (string->list "aeiou")) vowel?])
(add-eq-test 'tt-plus #f [$ every? "aeixou" vowel?])
(add-eq-test 'tt-plus #f [$ every? (string->list "aeixou") vowel?])
(add-eq-test 'tt-plus #f [$ every? (list->vector (string->list "aeixou")) vowel?])
(add-eq-test 'tt-plus #t [$ any? (list->vector (string->list "aeiou")) vowel?])
(add-eq-test 'tt-plus #f [$ any? (list->vector (string->list "aeiou")) nv?])
(add-eq-test 'tt-plus #t [$ any? (string->list "aeixou") nv?])
(add-eq-test 'tt-plus #f [$ any? (string->list "aeiou" ) nv?])
(add-eq-test 'tt-plus #t [$ any? "aeixou" nv?])
(add-eq-test 'tt-plus #f [$ any? "aeiou"  nv?])
(add-eq-test 'tt-plus #t [$ any? "aeixou" nv?])
(add-test 'tt-plus "ths s  lttl strng"
          [$ reject  "this is a little string" vowel?] string=?)
(add-test 'tt-plus "ths s  lttl strng"
          [$ collect "this is a little string" nv?]
          =?)
(add-test 'tt-plus (string->list "ths s  lttl strng")
          [$ reject (string->list "this is a little string") vowel?]
          equal?)
(add-test 'tt-plus (string->list "ths s  lttl strng")
          [$ collect (string->list "this is a little string") nv?]
          =?)
(add-test 'tt-plus (list->vector (string->list "ths s  lttl strng"))
          [$ reject  (list->vector (string->list "this is a little string")) vowel?]
          =?)
(add-test 'tt-plus (list->vector (string->list "ths s  lttl strng"))
          [$ collect (list->vector (string->list "this is a little string")) nv?]
          =?)
(add-test 'tt-plus "123"          [$ map "123"          id] =?)
(add-test 'tt-plus (vector 1 2 3) [$ map (vector 1 2 3) id] =?)
(add-test 'tt-plus (list   1 2 3) [$ map (list   1 2 3) id] =?)
(add-test 'tt-plus "23"         [$ slice "012345"             2 4] =?)
(add-test 'tt-plus (vector 2 3) [$ slice (vector 0 1 2 3 4 5) 2 4] =?)
(add-test 'tt-plus (list   2 3) [$ slice (list   0 1 2 3 4 5) 2 4] =?)
(add-test 'tt-plus 6 [$ length "012345"] =?)
(add-test 'tt-plus 6 [$ length (vector 0 1 2 3 4 5)] =?)
(add-test 'tt-plus 6 [$ length (list   0 1 2 3 4 5)] =?)
(add-test 'tt-plus #\3 [$ iref "012345"             3] =?)
(add-test 'tt-plus   3 [$ iref (vector 0 1 2 3 4 5) 3] =?)
(add-test 'tt-plus   3 [$ iref (list   0 1 2 3 4 5) 3] =?)
(add-test 'tt-plus "123"          [$ shallow-clone "123"] =?)
(add-test 'tt-plus (vector 1 2 3) [$ shallow-clone (vector 1 2 3)] =?)
(add-test 'tt-plus (list   1 2 3) [$ shallow-clone (list   1 2 3)] =?)

;==============================================================;
;;; VIRTUAL VARIABLES

(add-test-suite 'virtual-values default-setup-thunk default-teardown-thunk)

(define close-enough?
  (let ( [epsilon 0.0001] )
    (lambda (a b) (< (abs (- a b)) epsilon))))

(define cpx
  (object ([name 'complex][real 3][imag 4])
          [(->string self)
           (format #f "#<COMPLEX real:~a imag:~a>"
                   [$ real self][$ imag self])
           ]
) )
                    
[$ add-variable! cpx
   (cons 'angle
         (case-lambda
           [(self)
            (atan [$ imag self] [$ real self])
            ]
           [(self new-angle)
            (let ([magnitude [$ magnitude self]])
              [$ real self (* magnitude (cos new-angle))]
              [$ imag self (* magnitude (sin new-angle))]
              self)
            ])
) ]

[$ add-variable! cpx
   (cons 'magnitude
         (case-lambda
           [(self)
            (let ([r [$ real self]][i [$ imag self]])
              (sqrt (+ (* r r) (* i i))))
            ]
           [(self new-magnitude)
            (let ([angle [$ angle self]])
              [$ real self (* new-magnitude (cos angle))]
              [$ imag self (* new-magnitude (sin angle))]
              self)
            ])
) ]


(add-test 'virtual-values
          5
          (begin [$ real cpx 3] [$ imag cpx 4] [$ magnitude cpx])
          close-enough?)
(add-test 'virtual-values (atan 4 3) [$ angle cpx] close-enough?)
(add-test 'virtual-values
          6
          (begin [$ magnitude cpx 10][$ real cpx])
          close-enough?)
(add-test 'virtual-values 8 [$ imag cpx] close-enough?)
(add-test 'virtual-values
          8
          (begin [$ angle cpx (angle 8+6i)] [$ real cpx])
           close-enough?)


;==============================================================;

;; Run the Tests

(break-on-test-error? #t)
(verbose-test-output? #f)

(run-all-tests)

(newline)


;===========================E=O=F==============================;
